@@ -4,7 +4,15 @@ from __future__ import annotations
 
 from dataclasses import dataclass, field
 from functools import cached_property
-from typing import TYPE_CHECKING, ClassVar, Protocol, TypeVar, runtime_checkable
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    ClassVar,
+    Protocol,
+    TypeGuard,
+    TypeVar,
+    runtime_checkable,
+)
 
 from typing_extensions import Self
 
@@ -71,26 +79,40 @@ class Op(Protocol):
         return str(self)
 
 
-def _sig_port_type(sig: tys.FunctionType, port: InPort | OutPort) -> tys.Type:
-    """Get the type of the given dataflow port given the signature of the operation."""
-    if port.offset == -1:
-        # Order port
-        msg = "Order port has no type."
-        raise ValueError(msg)
-    if port.direction == Direction.INCOMING:
-        return sig.input[port.offset]
-    return sig.output[port.offset]
-
-
 @runtime_checkable
 class DataflowOp(Op, Protocol):
     """Abstract dataflow operation. Can be assumed to have a signature and Value-
     kind ports.
     """
 
+    #: When initializing a Hugr with a dataflow operation
+    #: a function is defined in the root module containing the op,
+    #: marked as entrypoint.
+    #: If the operation's output types are only known _after_ the
+    #: HUGR is defined, we need to wire up the function containing
+    #: the entrypoint as soon as the outputs are set.
+    #:
+    #: This flag is set to True for such cases. It should never be set
+    #: manually.
+    _entrypoint_requires_wiring: bool = field(
+        init=False, repr=False, default=False, compare=False
+    )
+
+    def _inputs(self) -> tys.TypeRow:
+        """The external input row of this operation. Defines the valid external
+        connectivity of the node the operation belongs to.
+
+        Raises:
+            IncompleteOp: If the operation's inputs have not been set.
+        """
+        ...  # pragma: no cover
+
     def outer_signature(self) -> tys.FunctionType:
         """The external signature of this operation. Defines the valid external
         connectivity of the node the operation belongs to.
+
+        Raises:
+            IncompleteOp: If the operation's inputs and outputs have not been set.
         """
         ...  # pragma: no cover
 
@@ -109,7 +131,17 @@ class DataflowOp(Op, Protocol):
             Bool
 
         """
-        return _sig_port_type(self.outer_signature(), port)
+        sig = self.outer_signature()
+        if port.offset == -1:
+            # Order port
+            msg = "Order port has no type."
+            raise ValueError(msg)
+        try:
+            if port.direction == Direction.INCOMING:
+                return sig.input[port.offset]
+            return sig.output[port.offset]
+        except IndexError as e:
+            raise self._invalid_port(port) from e
 
     def __call__(self, *args) -> Command:
         """Calling with incoming :class:`Wire` arguments returns a
@@ -119,6 +151,31 @@ class DataflowOp(Op, Protocol):
         return Command(self, list(args))
 
 
+def is_dataflow_op(op: Any) -> TypeGuard[DataflowOp]:
+    """Returns `true` if the object is an instance of :class:`DataflowOp`.
+
+    This is functionally equivalent to matching on `DataflowOp()` directly, but
+    calling `isinstance(_, DataflowOp)` errors out in `python <=3.11` due to how
+    runtime_checkable Protocols were implemented. See <https://github.com/python/cpython/issues/102433>
+    """
+    match op:
+        case (
+            Custom()
+            | Tag()
+            | DFG()
+            | CFG()
+            | LoadConst()
+            | Conditional()
+            | TailLoop()
+            | CallIndirect()
+            | LoadFunc()
+            | AsExtOp()
+        ):
+            return True
+        case _:
+            return False
+
+
 @runtime_checkable
 class _PartialOp(Protocol):
     def _set_in_types(self, types: tys.TypeRow) -> None: ...
@@ -180,6 +237,9 @@ class Input(DataflowOp):
     def _to_serial(self, parent: Node) -> sops.Input:
         return sops.Input(parent=parent.idx, types=ser_it(self.types))
 
+    def _inputs(self) -> tys.TypeRow:
+        return []
+
     def outer_signature(self) -> tys.FunctionType:
         return tys.FunctionType(input=[], output=self.types)
 
@@ -206,6 +266,9 @@ class Output(DataflowOp, _PartialOp):
     def _to_serial(self, parent: Node) -> sops.Output:
         return sops.Output(parent=parent.idx, types=ser_it(self.types))
 
+    def _inputs(self) -> tys.TypeRow:
+        return self.types
+
     def outer_signature(self) -> tys.FunctionType:
         return tys.FunctionType(input=self.types, output=[])
 
@@ -291,6 +354,9 @@ class AsExtOp(DataflowOp, Protocol):
             and slf.args == other.args
         )
 
+    def _inputs(self) -> tys.TypeRow:
+        return self.outer_signature().input
+
     def outer_signature(self) -> tys.FunctionType:
         return self.ext_op.outer_signature()
 
@@ -315,7 +381,6 @@ class Custom(DataflowOp):
 
     op_name: str
     signature: tys.FunctionType = field(default_factory=tys.FunctionType.empty)
-    description: str = ""
     extension: tys.ExtensionId = ""
     args: list[tys.TypeArg] = field(default_factory=list)
 
@@ -325,10 +390,12 @@ class Custom(DataflowOp):
             extension=self.extension,
             name=self.op_name,
             signature=self.signature._to_serial(),
-            description=self.description,
             args=ser_it(self.args),
         )
 
+    def _inputs(self) -> tys.TypeRow:
+        return self.signature.input
+
     def outer_signature(self) -> tys.FunctionType:
         return self.signature
 
@@ -407,6 +474,11 @@ class ExtOp(AsExtOp):
     def from_ext(cls, custom: ExtOp) -> ExtOp:
         return custom
 
+    def _inputs(self) -> tys.TypeRow:
+        if self.signature is None:
+            raise IncompleteOp(self)
+        return self.signature.input
+
     def outer_signature(self) -> tys.FunctionType:
         if self.signature is not None:
             return self.signature
@@ -447,6 +519,9 @@ class MakeTuple(AsExtOp, _PartialOp):
         """
         return _check_complete(self, self._types)
 
+    def _inputs(self) -> tys.TypeRow:
+        return self.types
+
     def op_def(self) -> ext.OpDef:
         from hugr import std  # no circular import
 
@@ -456,7 +531,6 @@ class MakeTuple(AsExtOp, _PartialOp):
         return tys.FunctionType(
             input=self.types,
             output=[tys.Tuple(*self.types)],
-            runtime_reqs=["prelude"],
         )
 
     def type_args(self) -> list[tys.TypeArg]:
@@ -499,7 +573,6 @@ class UnpackTuple(AsExtOp, _PartialOp):
         return tys.FunctionType(
             input=[tys.Tuple(*self.types)],
             output=self.types,
-            runtime_reqs=["prelude"],
         )
 
     def type_args(self) -> list[tys.TypeArg]:
@@ -512,6 +585,9 @@ class UnpackTuple(AsExtOp, _PartialOp):
     def __call__(self, tuple_: ComWire) -> Command:
         return super().__call__(tuple_)
 
+    def _inputs(self) -> tys.TypeRow:
+        return MakeTuple(self.types).outer_signature().output
+
     def outer_signature(self) -> tys.FunctionType:
         return MakeTuple(self.types).outer_signature().flip()
 
@@ -547,6 +623,9 @@ class Tag(DataflowOp):
             variants=[ser_it(r) for r in self.sum_ty.variant_rows],
         )
 
+    def _inputs(self) -> tys.TypeRow:
+        return self.sum_ty.variant_rows[self.tag]
+
     def outer_signature(self) -> tys.FunctionType:
         return tys.FunctionType(
             input=self.sum_ty.variant_rows[self.tag], output=[self.sum_ty]
@@ -625,6 +704,21 @@ class DfParentOp(Op, Protocol):
     def _inputs(self) -> tys.TypeRow: ...
 
 
+def is_df_parent_op(op: Any) -> TypeGuard[DfParentOp]:
+    """Returns `true` if the object is an instance of :class:`DfParentOp`.
+
+    This is functionally equivalent to matching on `DfParentOp()` directly, but
+    calling `isinstance(_, DfParentOp)` errors out in `python <=3.11` due to how
+    runtime_checkable Protocols were implemented.
+    See <https://github.com/python/cpython/issues/102433>
+    """
+    match op:
+        case DFG() | DataflowBlock() | Case() | TailLoop() | FuncDefn():
+            return True
+        case _:
+            return False
+
+
 @dataclass
 class DFG(DfParentOp, DataflowOp):
     """Simple dataflow graph operation. Outer signature matches inner signature."""
@@ -632,7 +726,6 @@ class DFG(DfParentOp, DataflowOp):
     #: Inputs types of the operation.
     inputs: tys.TypeRow
     _outputs: tys.TypeRow | None = field(default=None, repr=False)
-    _extension_delta: tys.ExtensionSet = field(default_factory=list, repr=False)
 
     @property
     def outputs(self) -> tys.TypeRow:
@@ -650,7 +743,7 @@ class DFG(DfParentOp, DataflowOp):
         Raises:
             IncompleteOp: If the outputs have not been set.
         """
-        return tys.FunctionType(self.inputs, self.outputs, self._extension_delta)
+        return tys.FunctionType(self.inputs, self.outputs)
 
     @property
     def num_out(self) -> int:
@@ -720,6 +813,9 @@ class CFG(DataflowOp):
     def name(self) -> str:
         return "CFG"
 
+    def _inputs(self) -> tys.TypeRow:
+        return self.inputs
+
 
 @dataclass
 class DataflowBlock(DfParentOp):
@@ -729,7 +825,6 @@ class DataflowBlock(DfParentOp):
     inputs: tys.TypeRow
     _sum: tys.Sum | None = None
     _other_outputs: tys.TypeRow | None = field(default=None, repr=False)
-    extension_delta: tys.ExtensionSet = field(default_factory=list)
 
     @property
     def sum_ty(self) -> tys.Sum:
@@ -762,7 +857,6 @@ class DataflowBlock(DfParentOp):
             inputs=ser_it(self.inputs),
             sum_rows=list(map(ser_it, self.sum_ty.variant_rows)),
             other_outputs=ser_it(self.other_outputs),
-            extension_delta=self.extension_delta,
         )
 
     def inner_signature(self) -> tys.FunctionType:
@@ -868,6 +962,9 @@ class LoadConst(DataflowOp):
             datatype=self.type_._to_serial_root(),
         )
 
+    def _inputs(self) -> tys.TypeRow:
+        return []
+
     def outer_signature(self) -> tys.FunctionType:
         return tys.FunctionType(input=[], output=[self.type_])
 
@@ -875,10 +972,8 @@ class LoadConst(DataflowOp):
         match port:
             case InPort(_, 0):
                 return tys.ConstKind(self.type_)
-            case OutPort(_, 0):
-                return tys.ValueKind(self.type_)
             case _:
-                raise self._invalid_port(port)
+                return DataflowOp.port_kind(self, port)
 
     def __repr__(self) -> str:
         return "LoadConst" + (f"({self._typ})" if self._typ is not None else "")
@@ -915,8 +1010,7 @@ class Conditional(DataflowOp):
         Raises:
             IncompleteOp: If the outputs have not been set.
         """
-        inputs = [self.sum_ty, *self.other_inputs]
-        return tys.FunctionType(inputs, self.outputs)
+        return tys.FunctionType(self._inputs(), self.outputs)
 
     @property
     def num_out(self) -> int:
@@ -942,6 +1036,10 @@ class Conditional(DataflowOp):
     def name(self) -> str:
         return "Conditional"
 
+    def _inputs(self) -> tys.TypeRow:
+        """Input row of the outer signature."""
+        return [self.sum_ty, *self.other_inputs]
+
 
 @dataclass
 class Case(DfParentOp):
@@ -993,7 +1091,6 @@ class TailLoop(DfParentOp, DataflowOp):
     #: Types that are appended to both inputs and outputs of the graph.
     rest: tys.TypeRow
     _just_outputs: tys.TypeRow | None = field(default=None, repr=False)
-    extension_delta: tys.ExtensionSet = field(default_factory=list, repr=False)
 
     @property
     def just_outputs(self) -> tys.TypeRow:
@@ -1014,7 +1111,6 @@ class TailLoop(DfParentOp, DataflowOp):
             just_inputs=ser_it(self.just_inputs),
             just_outputs=ser_it(self.just_outputs),
             rest=ser_it(self.rest),
-            extension_delta=self.extension_delta,
         )
 
     def inner_signature(self) -> tys.FunctionType:
@@ -1185,7 +1281,7 @@ class _CallOrLoad:
             self.type_args = list(type_args)
 
 
-class Call(_CallOrLoad, Op):
+class Call(_CallOrLoad, DataflowOp):
     """Call a function inside a dataflow graph. Connects to :class:`FuncDefn` or
     :class:`FuncDecl` operations.
 
@@ -1219,11 +1315,17 @@ class Call(_CallOrLoad, Op):
             case InPort(_, offset) if offset == self._function_port_offset():
                 return tys.FunctionKind(self.signature)
             case _:
-                return tys.ValueKind(_sig_port_type(self.instantiation, port))
+                return DataflowOp.port_kind(self, port)
 
     def name(self) -> str:
         return "Call"
 
+    def _inputs(self) -> tys.TypeRow:
+        return self.instantiation.input
+
+    def outer_signature(self) -> tys.FunctionType:
+        return self.instantiation
+
 
 @dataclass()
 class CallIndirect(DataflowOp, _PartialOp):
@@ -1255,6 +1357,10 @@ class CallIndirect(DataflowOp, _PartialOp):
     def __call__(self, function: ComWire, *args: ComWire) -> Command:  # type: ignore[override]
         return super().__call__(function, *args)
 
+    def _inputs(self) -> tys.TypeRow:
+        sig = self.signature
+        return [sig, *sig.input]
+
     def outer_signature(self) -> tys.FunctionType:
         sig = self.signature
 
@@ -1295,6 +1401,9 @@ class LoadFunc(_CallOrLoad, DataflowOp):
             instantiation=self.instantiation._to_serial(),
         )
 
+    def _inputs(self) -> tys.TypeRow:
+        return []
+
     def outer_signature(self) -> tys.FunctionType:
         return tys.FunctionType(input=[], output=[self.instantiation])
 
@@ -1302,10 +1411,8 @@ class LoadFunc(_CallOrLoad, DataflowOp):
         match port:
             case InPort(_, 0):
                 return tys.FunctionKind(self.signature)
-            case OutPort(_, 0):
-                return tys.ValueKind(self.instantiation)
             case _:
-                raise self._invalid_port(port)
+                return DataflowOp.port_kind(self, port)
 
     def name(self) -> str:
         return "LoadFunc"
@@ -1334,13 +1441,11 @@ class Noop(AsExtOp, _PartialOp):
     def cached_signature(self) -> tys.FunctionType | None:
         return tys.FunctionType.endo(
             [self.type_],
-            runtime_reqs=["prelude"],
         )
 
     def outer_signature(self) -> tys.FunctionType:
         return tys.FunctionType.endo(
             [self.type_],
-            runtime_reqs=["prelude"],
         )
 
     def _set_in_types(self, types: tys.TypeRow) -> None:
